const svg = d3.select('svg'); //select SVG element from d3.js resources
const projection = d3.geoMercator(); //To assign geoMercator() projection
let c_china = 0;
//To get json data from my gist
d3.json(
  'https://gist.githubusercontent.com/prathyushajampala/fb728d733943a87de85a75e4647e3207/raw/0b451bcdbbd5745b2c2a268feae6a857a225ceec/china.json',
  function (json) {
    const presimplify = topojson.presimplify(
      json
    ); //To perform simplification process to have efficient map
    const geojson = topojson.feature(
      presimplify,
      presimplify.objects
        .CHN_adm1
    ); //To convert topojson data into geojson data
  
    draw(geojson, c_china);  //To implement draw method
    const reDraw = draw.bind(null, geojson);//To implement redraw function with functionality as draw function
//To implement window resize so that the map exactly fits
    d3.select(window).on('resize', function () {
      reDraw(c_china);
    });
  }
);

//To Adjust window formats so that the map is clearly visible
function draw(geojson, threshold) {
  const w = svg.node().clientWidth;
  const h = svg.node().clientHeight;

  projection.fitExtent(
    [
      [0, 0],
      [w, h],
    ],
    geojson
  );

  //To Render json data 
  const simplify = d3.geoTransform({
    point: function (a, b, c) {
      //To convert into the pixel co-ordinates using projection function
      if (c >= threshold)
        this.stream.point(projection([a, b]));
    },
  });

  //To set path to SVG elements to visualize the China map
  const path = d3.geoPath().projection(simplify);
  //To bind data to svg so that we can visualize the updated data 
  const updateMaps = svg
    .selectAll('.map')
    .data([geojson]);
  const enterMaps = updateMaps
    .enter()
    .append('path')
    .attr('class', 'map');

  updateMaps.merge(enterMaps).attr('d', path);//To update map on web page for every change in geojson data
}
